/*
* Copyright (c) 2015, Freescale Semiconductor, Inc.
* Copyright 2016-2017 NXP
*
* Redistribution and use in source and binary forms, with or without modification,
* are permitted provided that the following conditions are met:
*
* o Redistributions of source code must retain the above copyright notice, this list
*   of conditions and the following disclaimer.
*
* o Redistributions in binary form must reproduce the above copyright notice, this
*   list of conditions and the following disclaimer in the documentation and/or
*   other materials provided with the distribution.
*
* o Neither the name of the copyright holder nor the names of its
*   contributors may be used to endorse or promote products derived from this
*   software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
* DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
* ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
* ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#include "fsl_debug_console.h"

#include "board.h"
#include "math.h"
#include "non_block_delay.h"

#include "composite.h"
#include "fsl_lptmr.h"


#include "fsl_mma.h"
#include "fsl_i2c.h"
#include "fsl_tpm.h"

#include "pin_mux.h"
#include "fsl_gpio.h"
#include "fsl_port.h"
/*******************************************************************************
* Definitions
******************************************************************************/
/* Get source clock for LPTMR driver */
#define LPTMR_SOURCE_CLOCK CLOCK_GetFreq(kCLOCK_LpoClk)
/*LPTMR period */
#define LPTMR_USEC_COUNT 1000U /*in uS*/


/* The TPM instance/channel used for board */
#define BOARD_TIMER_BASEADDR TPM2
#define BOARD_FIRST_TIMER_CHANNEL 0U
#define BOARD_SECOND_TIMER_CHANNEL 1U
/* Get source clock for TPM driver */
#define BOARD_TIMER_SOURCE_CLOCK CLOCK_GetFreq(kCLOCK_BusClk)
#define TIMER_CLOCK_MODE 1U
/* I2C source clock */
#define ACCEL_I2C_CLK_SRC I2C0_CLK_SRC
#define I2C_BAUDRATE 100000U

#define I2C_RELEASE_SDA_PORT PORTE
#define I2C_RELEASE_SCL_PORT PORTE
#define I2C_RELEASE_SDA_GPIO GPIOE
#define I2C_RELEASE_SDA_PIN 25U
#define I2C_RELEASE_SCL_GPIO GPIOE
#define I2C_RELEASE_SCL_PIN 24U
#define I2C_RELEASE_BUS_COUNT 100U

#define ANGLE_THRESHOLD 800 //THRESHOLD for "zero" movement of cursor

/* Upper bound and lower bound angle values */
#define ANGLE_UPPER_BOUND 85U
#define ANGLE_LOWER_BOUND 15U

extern volatile uint8_t dirUD, dirLR;
extern uint8_t key;
extern uint8_t mouse_button;
/*******************************************************************************
* Prototypes
******************************************************************************/
void BOARD_I2C_ReleaseBus(void);

/*******************************************************************************
* Variables
******************************************************************************/
i2c_master_handle_t g_MasterHandle;
/* MMA8451 device address */
const uint8_t g_accel_address[] = {0x1CU, 0x1DU, 0x1EU, 0x1FU};

/*******************************************************************************
* Code
******************************************************************************/
void LPTMR0_IRQHandler(void)
{
    LPTMR_ClearStatusFlags(LPTMR0, kLPTMR_TimerCompareFlag);

    tick_counter++; // non blocking delay utility tisk every 1mS

};
static void i2c_release_bus_delay(void)
{
    uint32_t i = 0;
    for (i = 0; i < I2C_RELEASE_BUS_COUNT; i++)
    {
        __NOP();
    }
}

void BOARD_I2C_ReleaseBus(void)
{
    uint8_t i = 0;
    gpio_pin_config_t pin_config;
    port_pin_config_t i2c_pin_config = {0};

    /* Config pin mux as gpio */
    i2c_pin_config.pullSelect = kPORT_PullUp;
    i2c_pin_config.mux = kPORT_MuxAsGpio;

    pin_config.pinDirection = kGPIO_DigitalOutput;
    pin_config.outputLogic = 1U;
    CLOCK_EnableClock(kCLOCK_PortE);
    PORT_SetPinConfig(I2C_RELEASE_SCL_PORT, I2C_RELEASE_SCL_PIN, &i2c_pin_config);
    PORT_SetPinConfig(I2C_RELEASE_SDA_PORT, I2C_RELEASE_SDA_PIN, &i2c_pin_config);

    GPIO_PinInit(I2C_RELEASE_SCL_GPIO, I2C_RELEASE_SCL_PIN, &pin_config);
    GPIO_PinInit(I2C_RELEASE_SDA_GPIO, I2C_RELEASE_SDA_PIN, &pin_config);

    /* Drive SDA low first to simulate a start */
    GPIO_WritePinOutput(I2C_RELEASE_SDA_GPIO, I2C_RELEASE_SDA_PIN, 0U);
    i2c_release_bus_delay();

    /* Send 9 pulses on SCL and keep SDA high */
    for (i = 0; i < 9; i++)
    {
        GPIO_WritePinOutput(I2C_RELEASE_SCL_GPIO, I2C_RELEASE_SCL_PIN, 0U);
        i2c_release_bus_delay();

        GPIO_WritePinOutput(I2C_RELEASE_SDA_GPIO, I2C_RELEASE_SDA_PIN, 1U);
        i2c_release_bus_delay();

        GPIO_WritePinOutput(I2C_RELEASE_SCL_GPIO, I2C_RELEASE_SCL_PIN, 1U);
        i2c_release_bus_delay();
        i2c_release_bus_delay();
    }

    /* Send stop */
    GPIO_WritePinOutput(I2C_RELEASE_SCL_GPIO, I2C_RELEASE_SCL_PIN, 0U);
    i2c_release_bus_delay();

    GPIO_WritePinOutput(I2C_RELEASE_SDA_GPIO, I2C_RELEASE_SDA_PIN, 0U);
    i2c_release_bus_delay();

    GPIO_WritePinOutput(I2C_RELEASE_SCL_GPIO, I2C_RELEASE_SCL_PIN, 1U);
    i2c_release_bus_delay();

    GPIO_WritePinOutput(I2C_RELEASE_SDA_GPIO, I2C_RELEASE_SDA_PIN, 1U);
    i2c_release_bus_delay();
}
/* Initialize timer module */
static void Timer_Init(void)
{
    /* convert to match type of data */
    tpm_config_t tpmInfo;
    tpm_chnl_pwm_signal_param_t tpmParam[2];

    /* Configure tpm params with frequency 24kHZ */
    tpmParam[0].chnlNumber = (tpm_chnl_t)BOARD_FIRST_TIMER_CHANNEL;
    tpmParam[0].level = kTPM_LowTrue;
    tpmParam[0].dutyCyclePercent = 0U;

    tpmParam[1].chnlNumber = (tpm_chnl_t)BOARD_SECOND_TIMER_CHANNEL;
    tpmParam[1].level = kTPM_LowTrue;
    tpmParam[1].dutyCyclePercent = 0U;

    /* Initialize TPM module */
    TPM_GetDefaultConfig(&tpmInfo);
    TPM_Init(BOARD_TIMER_BASEADDR, &tpmInfo);

    CLOCK_SetTpmClock(1U);

    TPM_SetupPwm(BOARD_TIMER_BASEADDR, tpmParam, 2U, kTPM_EdgeAlignedPwm, 24000U, BOARD_TIMER_SOURCE_CLOCK);
    TPM_StartTimer(BOARD_TIMER_BASEADDR, kTPM_SystemClock);
}

/* Update the duty cycle of an active pwm signal */
static void Board_UpdatePwm(uint16_t x, uint16_t y)
{
    /* Updated duty cycle */
    TPM_UpdatePwmDutycycle(BOARD_TIMER_BASEADDR, (tpm_chnl_t)BOARD_FIRST_TIMER_CHANNEL, kTPM_EdgeAlignedPwm, x);
    TPM_UpdatePwmDutycycle(BOARD_TIMER_BASEADDR, (tpm_chnl_t)BOARD_SECOND_TIMER_CHANNEL, kTPM_EdgeAlignedPwm, y);
}

int main(void)
{
    static uint32_t delay1;
    static uint32_t delay_button_sample =200;//200mS

    mma_handle_t mmaHandle = {0};
    mma_data_t sensorData = {0};
    i2c_master_config_t i2cConfig = {0};
    uint8_t sensorRange = 0;
    uint8_t dataScale = 0;
    uint32_t i2cSourceClock = 0;
    int16_t xData = 0;
    int16_t yData = 0;
    int16_t xAngle = 0;
    int16_t yAngle = 0;
    uint8_t i = 0;
    uint8_t regResult = 0;
    uint8_t array_addr_size = 0;
    bool foundDevice = false;

    /* Board pin, clock, debug console init */
    BOARD_InitPins();
    BOARD_BootClockRUN();
    BOARD_I2C_ReleaseBus();
    BOARD_I2C_ConfigurePins();
    //BOARD_InitDebugConsole();

    i2cSourceClock = CLOCK_GetFreq(ACCEL_I2C_CLK_SRC);
    mmaHandle.base = BOARD_ACCEL_I2C_BASEADDR;
    mmaHandle.i2cHandle = &g_MasterHandle;

    I2C_MasterGetDefaultConfig(&i2cConfig);
    I2C_MasterInit(BOARD_ACCEL_I2C_BASEADDR, &i2cConfig, i2cSourceClock);
    I2C_MasterTransferCreateHandle(BOARD_ACCEL_I2C_BASEADDR, &g_MasterHandle, NULL, NULL);

    /* Find sensor devices */
    array_addr_size = sizeof(g_accel_address) / sizeof(g_accel_address[0]);
    for (i = 0; i < array_addr_size; i++)
    {
        mmaHandle.xfer.slaveAddress = g_accel_address[i];
        if (MMA_ReadReg(&mmaHandle, kMMA8451_WHO_AM_I, &regResult) == kStatus_Success)
        {
            foundDevice = true;
            break;
        }
        if ((i == (array_addr_size - 1)) && (!foundDevice))
        {
            PRINTF("\r\nDo not found sensor device\r\n");
            while (1)
            {
            };
        }
    }

    /* Init accelerometer sensor */
    if (MMA_Init(&mmaHandle) != kStatus_Success)
    {
        return -1;
    }

    /* Get sensor range */
    if (MMA_ReadReg(&mmaHandle, kMMA8451_XYZ_DATA_CFG, &sensorRange) != kStatus_Success)
    {
        return -1;
    }
    if (sensorRange == 0x00)
    {
        dataScale = 2U;
    }
    else if (sensorRange == 0x01)
    {
        dataScale = 4U;
    }
    else if (sensorRange == 0x10)
    {
        dataScale = 8U;
    }
    else
    {
    }
    /* Init timer */
   // Timer_Init();
    USB_DeviceApplicationInit();


    /* Configure LPTMR */
    /*
    * lptmrConfig.timerMode = kLPTMR_TimerModeTimeCounter;
    * lptmrConfig.pinSelect = kLPTMR_PinSelectInput_0;
    * lptmrConfig.pinPolarity = kLPTMR_PinPolarityActiveHigh;
    * lptmrConfig.enableFreeRunning = false;
    * lptmrConfig.bypassPrescaler = true;
    * lptmrConfig.prescalerClockSource = kLPTMR_PrescalerClock_1;
    * lptmrConfig.value = kLPTMR_Prescale_Glitch_0;
    */
    lptmr_config_t lptmrConfig;
    LPTMR_GetDefaultConfig(&lptmrConfig);

    /* Initialize the LPTMR */
    LPTMR_Init(LPTMR0, &lptmrConfig);

    /*
    * Set timer period.
    * Note : the parameter "ticks" of LPTMR_SetTimerPeriod should be equal or greater than 1.
    */
    LPTMR_SetTimerPeriod(LPTMR0, USEC_TO_COUNT(LPTMR_USEC_COUNT, LPTMR_SOURCE_CLOCK));

    /* Enable timer interrupt */
    LPTMR_EnableInterrupts(LPTMR0, kLPTMR_TimerInterruptEnable);

    /* Enable at the NVIC */
    EnableIRQ(LPTMR0_IRQn);

    uint32_t mouse_release = 1;
    uint32_t keyboard_release = 1;

    /* Start counting */
    LPTMR_StartTimer(LPTMR0);

    /*START delay measurement for main button sampling*/
    delay1= tick_counter;
    /* Main loop. Get sensor data and update duty cycle */
    while (1)
    {


        /* Get new accelerometer data. */
        if (MMA_ReadSensorData(&mmaHandle, &sensorData) != kStatus_Success)
        {
            return -1;
        }

/* Get the X and Y data from the sensor data structure in 14 bit left format data*/
xData = (int16_t)((uint16_t)((uint16_t)sensorData.accelXMSB << 8) | (uint16_t)sensorData.accelXLSB) / 4U;
yData = (int16_t)((uint16_t)((uint16_t)sensorData.accelYMSB << 8) | (uint16_t)sensorData.accelYLSB) / 4U;



if (xData > ANGLE_THRESHOLD)
{
    dirUD = DOWN;
}else if (xData < -ANGLE_THRESHOLD)
{
    dirUD = UP;
}else
{
    dirUD=STOP;
}

if (yData > ANGLE_THRESHOLD)
{
    dirLR = LEFT;
}else if (yData < -ANGLE_THRESHOLD)
{
    dirLR = RIGHT;
}else
    dirLR = STOP;


//        /* Convert raw data to angle (normalize to 0-90 degrees). No negative angles. */
//        xAngle = (int16_t)floor((double)xData * (double)dataScale * 90 / 8192);
//        if (xAngle < 0)
//        {
//            xAngle *= -1;
//        }
//        yAngle = (int16_t)floor((double)yData * (double)dataScale * 90 / 8192);
//        if (yAngle < 0)
//        {
//            yAngle *= -1;
//        }
//        /* Update angles to turn on LEDs when angles ~ 90 */
//        if (xAngle > ANGLE_UPPER_BOUND)
//        {
//            xAngle = 100;
//        }
//        if (yAngle > ANGLE_UPPER_BOUND)
//        {
//            yAngle = 100;
//        }
//        /* Update angles to turn off LEDs when angles ~ 0 */
//        if (xAngle < ANGLE_LOWER_BOUND)
//        {
//            xAngle = 0;
//        }
//        if (yAngle < ANGLE_LOWER_BOUND)
//        {
//            yAngle = 0;
//        }
//
//        Board_UpdatePwm(xAngle, yAngle);

        if (is_time_now(true, &delay1, delay_button_sample))
        {
            if (!GPIO_ReadPinInput(PLUG_BOARD_GPIO, SW_PREV_PIN))
            {// previsou slide
            if (keyboard_release){ // detect release
                key = KEY_LEFTARROW;
                keyboard_release = 0;
                }else
                    key = STOP;

            }else if (!GPIO_ReadPinInput(PLUG_BOARD_GPIO, SW_NEXT_PIN))
            {// next slide
            if (keyboard_release){ // detect release
                key = KEY_RIGHTARROW;
                keyboard_release = 0;
                }else
                    key = STOP;

            } else if (!GPIO_ReadPinInput(PLUG_BOARD_GPIO, SW_GO_PRES_PIN))
            {// Presentation ON
              if (keyboard_release){ // detect release
                    key = KEY_F5;
                keyboard_release = 0;
                }else
                    key = STOP;
            }else if (!GPIO_ReadPinInput(PLUG_GO_BLACK_GPIO, SW_GO_BLACK_PIN))
            {//Blackout
                if (keyboard_release){ // detect release
                    key = KEY_B;
                keyboard_release = 0;
                }else
                    key = STOP;
            } else
                {
                keyboard_release = 1;
                key = STOP;
                }
            if (!GPIO_ReadPinInput(PLUG_BOARD_GPIO, SW_LEFT_BUTTON_PIN))
            {//Pointing function
                if(mouse_release)
                {
                    mouse_release = 0;
                    if( mouse_button == LEFT)
                       {
                         mouse_button = STOP;
                       }
                    else
                        mouse_button = LEFT;
                }
            }else
            {
            mouse_release = 1;
            }

        }
    }
}
